;+
; NAME :
;   lst_fitshead2struct.pro (function)
; PURPOSE :
;   convert FITS header->struct. It overcomes the problems of FITSHEAD2STRUCT.PRO, which changes datatype and adds COMMENT & HISTORY.   
; PROJECT :
;   ASO-S / LST
; CATEGORY :
;   Data processing
; CALLING SEQUENCE :
;   hdr_struct = lst_fitshead2struct(head, ncomment=20, nhistory=20, status=status)
; INPUTS :
;   head : string array or structure containing FITS header. If head is a struture, only the number of COMMENT and/or HISTORY is changed
;          with ncomment and/or nhistory set. 
; OPTIONAL INPUT PARAMETERS :
;   ncomment : element number of tag comment of the output header.
;   nhistory : element number of tag history of the output header.
; OUTPUTS :
;   hdr_struct: a structure-type header.
; OPTIONAL OUTPUT PARAMETERS :
;   status: a variable for result status. 1 = successfully get the conversion factor; -1 = failed to get the factors
; KEYWORD PARAMETERS :
;   silent: if set, no infomation is printed.
; COMMON BLOCKS :
;   None
; SIDE EFFECTS :
;   If the number of "COMMENT" or "HISTORY" of the input header, excluding empty strings, exceeds ncomment or nhistory, then exceeded items are dropped. 
; RESTRICTIONS :
;   None
; PROCEDURE :
;   called: DATA_CHK, TAG_EXIST, REM_TAG, ADD_TAG, STR_REPLACE, FXPAR, STRARRCOMPRESS, STRSPECIAL, ID_ESC, SSW_UNIQ, REM_ELEM, PRSTR,
;           STRJUSTIFY, FMT_TAG, MRD_STRUCT, DATATYPE, IS_NUMBER, BOOST_TAG, TEMPORARY
; EXAMPLE :
;   hdr_struct = lst_fitshead2struct(hdr, ncomment=20, nhistory=20, status=status)
; MODIFICATION HISTORY :
;   2022/06/10 Jianchao Xue, created.
;   2022/06/13 Jianchao Xue, non empty COMMENTs and HISTORYs in the input header are used to fill the new header before empty strings. 
;------------------------------------------------------------------------
;-
function lst_set_nstr, sarr, num, info=info, imodify=imodify
; return a string array with "num" elements on the basis of sarr

 imodify = 1
 ind = where(sarr ne '', nsarr)
 if nsarr gt 0 then sarr = sarr[ind]
 ;nsarr = n_elements(sarr)
 case 1 of
  nsarr gt num: begin
    res = sarr[0:num-1]
    if keyword_set(info) then message, /info, info
    end
  nsarr eq num: begin
    res = sarr
    imodify = 0
    end
  else: begin
    res = strarr(num) ; replicate('', num)
    if nsarr gt 0 then res[0:nsarr-1] = sarr
    end
 endcase
 return, res
end

function lst_fitshead2struct, head, ncomment=ncomment, nhistory=nhistory, status=status, silent=silent
; the main function

 status=-1
 
 ; ------------------ Check the input ----------------------------
 if data_chk(head,/struct) then begin
  imodify = 0
  if keyword_set(ncomment) then begin
    if tag_exist(head, 'comment') then begin
      coms = head.comment
      head = rem_tag(head, 'comment')
      if keyword_set(silent) then info=0 else info="LST_FITSHEAD2STRUCT: The number of comments is larger than "+strtrim(string(ncomment), 2)+" and is truncated." 
      coms_new = lst_set_nstr(coms, ncomment, info=info, imodify=imsub)
      head = add_tag(head, coms_new, 'comment')
      imodify += imsub
    endif else begin
      head = add_tag(head, strarr(ncomment), 'comment')
      imodify += 1
    endelse
  endif
  if keyword_set(nhistory) then begin
    if tag_exist(head, 'history') then begin
      hists = head.history
      head = rem_tag(head, 'history')
      if keyword_set(silent) then info=0 else info="LST_FITSHEAD2STRUCT: The number of histories is larger than "+strtrim(string(nhistory), 2)+" and is truncated."
      hists_new = lst_set_nstr(hists, nhistory, info=info, imodify=imsub)
      head = add_tag(head, hists_new, 'history')
      imodify += imsub
    endif else begin
      head = add_tag(head, strarr(nhistory), 'history')
      imodify += 1
    endelse
  endif
  if imodify eq 0 then $
    if not keyword_set(silent) then $
      message, /info, 'No need to change the input header.'
  status = 1
  return, head
 endif else begin
  if not data_chk(head,/string)  then begin
    message,/info,"Nead FITS header as input"
    return, ''
  endif
 endelse
 
 ; -------- convert string array to sturcture ------------
 ohead=head                           ; dont corrupt input

 ; -------- protect against FITS rules violation -------------
 badch=where(strpos(strupcase(strcompress(ohead,/remove)),'COMMENT=') eq 0 or $
   strpos(strupcase(strcompress(ohead,/remove)),'HISTORY=') eq 0,bchcnt)
 if bchcnt gt 0 then begin
   IF ~keyword_set(SILENT) THEN message,/info,"Correcting COMMENT or HISTORY record rule violation"
   ohead(badch)=str_replace(ohead(badch),'=',' ')
 endif
 ; --------------------------------------------------------------------
 
 ; -------------- some useful cleanup --------------------------------
 deblank=strarrcompress(ohead)        ; version with no null lines
 ;comp=strcompress(deblank,/remove)    ; compressed version, if used, the datatype is changed
 epos=strpos(deblank,'=')                             ; 
 ;

 keys=where( epos ge 1 and epos le 9 and (1-strspecial(deblank)),kcnt)
 keys=strcompress(strmid(deblank[keys], 0, 8), /remove)
 tags=id_esc(keys)
 utags=ssw_uniq(tags,sort(tags),/first)
 ntag=n_elements(utags)
 
 ;  -------- eliminate duplicate tags -------           ; DATE_OBS DATE-OBS...
 if ntag lt kcnt then begin
   atags=lindgen(kcnt)
   dtags=rem_elem(atags,utags)
   IF ~keyword_set(SILENT) THEN prstr,strjustify(['Duplicate Tags: (only preserving first occurence)',$
        '  ' + tags(dtags)],/center,/box)
 ;  ---------------   update ----------------
   order=utags(sort(utags))
   keys=keys(order)
   tags=tags(order)
   kcnt=ntag
 ;  -----------------------------------------
 endif
  
 ;  -------- get COMMENT & HISTORY --------------------------------
 comments=fxpar(ohead,'COMMENT',count=ccnt)
 history =fxpar(ohead,'HISTORY',count=hcnt)
 
 ;  -------- create the structure --------------------------------
 if not keyword_set(ncomment) then ncomment=ccnt
 if not keyword_set(nhistory) then nhistory=hcnt
 structarr=strarr(kcnt)
 for i=0,kcnt-1 do structarr[i]=fmt_tag(size(fxpar(ohead,keys[i])))
 case 1 of
  ncomment gt 0 and nhistory gt 0: retval=mrd_struct([tags,'COMMENT','HISTORY'],[structarr,'strarr('+string(ncomment)+')','strarr('+string(nhistory)+')'],1)
  ncomment gt 0: retval=mrd_struct([tags,'COMMENT'],[structarr,'strarr('+string(ncomment)+')'],1)
  nhistory gt 0: retval=mrd_struct([tags,'HISTORY'],[structarr,'strarr('+string(nhistory)+')'],1)
  else: retval=mrd_struct(tags,structarr,1)
 endcase
 
 string_tags=intarr(kcnt)                             ; track string tags
 not_found=intarr(kcnt)
 
 ;  ---------------- fill the structure -------------
 for i=0,kcnt-1 do  begin
  tmp=fxpar(ohead,keys[i],count=fcount)
  dtype=datatype(retval.(i),2)
  ok=1
  if (dtype ge 1) and (dtype le 5) then ok=is_number(tmp)
  case 1 of
    n_elements(retval.(i)) eq 1 and n_elements(tmp) eq 1: retval.(i)=tmp(0)
    n_elements(tmp) gt n_elements(retval.(i)): $
      retval=boost_tag(temporary(retval),tmp,keys(i))
    ok: retval.(i)=tmp
  endcase
  string_tags(i)=data_chk(retval.(i),/string)       ; boolean STRINGS?
 endfor
 ; trim string tags                 ; default?? /notrim keyword??
 string_tags=where(string_tags,stcnt)
 for i=0,stcnt-1 do retval.(string_tags(i))=strtrim(retval.(string_tags(i)),2)
 
 if ncomment gt 0 then begin
  if keyword_set(silent) then info=0 else info="LST_FITSHEAD2STRUCT: The number of comments is larger than "+strtrim(string(ncomment), 2)+" and is truncated."
  coms_new = lst_set_nstr(comments, ncomment, info=info)
  retval.COMMENT = coms_new
 endif
 if nhistory gt 0 then begin
  if keyword_set(silent) then info=0 else info="LST_FITSHEAD2STRUCT: The number of histories is larger than "+strtrim(string(nhistory), 2)+" and is truncated."
  hists_new = lst_set_nstr(history, nhistory, info=info, imodify=imsub)
  retval.HISTORY = hists_new
 endif
 
 status = 1
 return, retval
end