;+
; NAME : 
;   lst_gen_hdr_cal.pro (procedure)
; PURPOSE :
;   Calculate some parameters from inputted image and file header, such as minimum, maximum, mean, 
;   median, skewness and so on. In the meantime, update these items in hdr.
; PROJECT :
;   ASO-S / LST
; CATEGORY :
;   Data processing
; CALLING SEQUENCE :
;   lst_gen_hdr_cal,hdr,inputImg,outputHdr=outputHdr,DATAZER=DATAZER,DATASAT=DATASAT,DATAMIN=DATAMIN,$
;   DATAMAX=DATAMAX,DATAMEDN=DATAMEDN,DATAMEAN=DATAMEAN,DATATOT=DATATOT,DATASIG=DATASIG,$
;   DATASKEW=DATASKEW,DATAKURT=DATAKURT,DATAP01=DATAP01,DATAP10=DATAP10,$
;   DATAP25=DATAP25,DATAP75=DATAP75,DATAP90=DATAP90,DATAP95=DATAP95,$
;   DATAP98=DATAP98,DATAP99=DATAP99,QUALITY=QUALITY
; INPUTS :
;   hdr = the file header of the inputted image
;   inputImg = a data array with 2 dimensions (2D)
; OPTIONAL INPUT PARAMETERS :
;   None
; OUTPUTS :
;   None
; OPTIONAL OUTPUT PARAMETERS :
;   outputHdr: the updated fits head
;   DATAZER: the number of zero pixels in the image
;   DATASAT: the number of saturated pixels in the image
;   DATAMIN: the minimum value of the image
;   DATAMAX: the maximum value of the image
;   DATAMEDN: the median value of the image
;   DATAMEAN: the mean value of the image
;   DATATOT: the total value of the image
;   DATASIG: the standard deviation of the image
;   DATASKEW: the Skewness of the image
;   DATAKURT: the Kurtosis of the image
;   DATAP01: pixel value corresponding to lowest 1 percentile
;   DATAP10: pixel value corresponding to lowest 10 percentile
;   DATAP25: pixel value corresponding to lowest 25 percentile
;   DATAP75: pixel value corresponding to lowest 75 percentile
;   DATAP90: pixel value corresponding to lowest 90 percentile
;   DATAP95: pixel value corresponding to lowest 95 percentile
;   DATAP98: pixel value corresponding to lowest 98 percentile
;   DATAP99: pixel value corresponding to lowest 99 percentile
;   QUALITY: bit flags for various kinds of badness
; KEYWORD PARAMETERS :
;   None
; COMMON BLOCKS :   
;   None
; SIDE EFFECTS :
;   None
; RESTRICTIONS :
;   Usable for arrays with 2D and its header.
; PROCEDURE :
;   PERCENTILES()
; EXAMPLE :
;   lst_gen_hdr_cal,hdr,inputImg,outputHdr=outputHdr,DATAZER=DATAZER,DATASAT=DATASAT
;   lst_gen_hdr_cal,hdr,inputImg,outputHdr=outputHdr,DATAP01=DATAP01,DATAP10=DATAP10
; MODIFICATION HISTORY :
;   2021/1/12 De-Chao Song (dcsong@pmo.ac.cn)   
;   2021/1/13 De-Chao Song    
;   2021/1/15 De-Chao Song 
;   2021/6/10 De-Chao Song  only calculate the saturated pixel number of level 1 data
;   2021/8/30 De-Chao Song  exclude the '+/-inf' and '+/-nan' data in the calculation
;   2021/12/20 De-Chao Song  add the 'datatot' parameter
;   2022/3/21 De-Chao Song  add the 'structure' parameter
;   2022/9/2 De-Chao Song  remove the 'structure' parameter and alter the format of input and output to "structure"

;------------------------------------------------------------------------
;-

pro lst_gen_hdr_cal,hdr,inputImg,outputHdr=outputHdr,DATAZER=DATAZER,DATASAT=DATASAT,DATAMIN=DATAMIN,$
DATAMAX=DATAMAX,DATAMEDN=DATAMEDN,DATAMEAN=DATAMEAN,DATATOT=DATATOT,DATASIG=DATASIG,$
DATASKEW=DATASKEW,DATAKURT=DATAKURT,DATAP01=DATAP01,DATAP10=DATAP10,$
DATAP25=DATAP25,DATAP75=DATAP75,DATAP90=DATAP90,DATAP95=DATAP95,$
DATAP98=DATAP98,DATAP99=DATAP99,QUALITY=QUALITY,status=status,_EXTRA=EXTRA_KEYWORDS

status=0
hdr0=hdr

IF datatype(hdr0) NE 'STC' then hdr0=lst_fitshead2struct(hdr0)

;Check the dimension of inputData.
dsize=size(inputImg)
IF dsize[0] NE 2 THEN BEGIN
  message,'Dimension of inputData is not right. Please input a 2D array.',/continue
  status=-1
  return
ENDIF

help,inputImg
;remove the bad and missing values
remove_ind_and_nan=where(finite(inputImg),nremove)

IF nremove EQ 0 THEN BEGIN 
  message,'InputData is bad. Please check it.',/continue
  status=-1
  return
ENDIF

tem_img=inputImg(remove_ind_and_nan)
w_badAndMissing=where((tem_img EQ -32768) OR (tem_img EQ -32767),complement=normal,/null)

IF normal EQ !null THEN BEGIN 
  message,'InputData is bad. Please check it.',/continue
  status=-1
  return
ENDIF

img=tem_img(normal)
;help,img
wDATAZER=where(img EQ 0,DATAZER)

wDATASAT=where(img GE hdr0.dsatval,DATASAT)


IF hdr0.camera eq 'SDI' OR hdr0.camera eq 'WST' AND strmid(hdr0.filename,7,2) EQ '00' then begin
if tag_exist(hdr0,'DATASAT') then hdr0.DATASAT=long(DATASAT) else hdr0=create_struct('DATASAT',long(DATASAT),hdr0)
ENDIF

IF hdr0.camera eq 'SCIUV' OR hdr0.camera eq 'SCIWL' AND strmid(hdr0.filename,9,2) EQ '00' then begin
if tag_exist(hdr0,'DATASAT') then hdr0.DATASAT=long(DATASAT) else hdr0=create_struct('DATASAT',long(DATASAT),hdr0)
ENDIF


; IF strmid(hdr0.filename,7,2) EQ '00' and hdr0.camera eq 'SDI' THEN BEGIN 
; ENDIF

; IF strmid(hdr0.filename,7,2) EQ '00' and hdr0.camera eq 'WST' THEN BEGIN 
; if tag_exist(hdr0,'DATASAT') then hdr0.DATASAT=DATASAT else hdr0=create_struct('DATASAT',DATASAT,hdr0)
; ENDIF

; IF strmid(hdr0.filename,9,2) EQ '00' and hdr0.camera eq 'SCIUV' THEN BEGIN 
; if tag_exist(hdr0,'DATASAT') then hdr0.DATASAT=DATASAT else hdr0=create_struct('DATASAT',DATASAT,hdr0)
; ENDIF

; IF strmid(hdr0.filename,9,2) EQ '00' and hdr0.camera eq 'SCIWL' THEN BEGIN 
; if tag_exist(hdr0,'DATASAT') then hdr0.DATASAT=DATASAT else hdr0=create_struct('DATASAT',DATASAT,hdr0)
; ENDIF


DATAMIN=float(min(img))
DATAMAX=float(max(img))
;help,img
;print,minmax(img)
DATAMEDN=median(img)
DATAMEAN=mean(img,/nan)
DATATOT=total(img,/nan)
mm=moment(img,/nan,/double);moment
DATASIG=sqrt(mm[1])
;PRINT,DATASIG
;PRINT,STDDEV(IMG)
DATASKEW=mm[2]
DATAKURT=mm[3]

pp=float((percentiles(img,value=[0.01,0.1,0.25,0.75,0.9,0.95,0.98,0.99])))
DATAP01=pp[0]
DATAP10=pp[1]
DATAP25=pp[2]
DATAP75=pp[3]
DATAP90=pp[4]
DATAP95=pp[5]
DATAP98=pp[6]
DATAP99=float(pp[7])

QUALITY=0
IF hdr0.percentd lt 50. THEN QUALITY=1



; PRINT,DATAZER
; PRINT,DATASAT
; PRINT,DATAMIN
; PRINT,DATAMAX
; PRINT,DATAMEDN
; PRINT,DATAMEAN
; PRINT,DATASIG
; PRINT,DATASKEW
; PRINT,DATAKURT
; PRINT,DATAP01
; PRINT,DATAP10
; PRINT,DATAP25
; PRINT,DATAP75
; PRINT,DATAP90
; PRINT,DATAP95
; PRINT,DATAP98
; PRINT,DATAP99
; PRINT,QUALITY


if tag_exist(hdr0,'DATAZER') then hdr0.DATAZER=DATAZER else hdr0=create_struct('DATAZER',DATAZER,hdr0)
if tag_exist(hdr0,'DATAMIN') then hdr0.DATAMIN=DATAMIN else hdr0=create_struct('DATAMIN',DATAMIN,hdr0)
if tag_exist(hdr0,'DATAMAX') then hdr0.DATAMAX=DATAMAX else hdr0=create_struct('DATAMAX',DATAMAX,hdr0)
if tag_exist(hdr0,'DATAMEDN') then hdr0.DATAMEDN=DATAMEDN else hdr0=create_struct('DATAMEDN',DATAMEDN,hdr0)
if tag_exist(hdr0,'DATAMEAN') then hdr0.DATAMEAN=DATAMEAN else hdr0=create_struct('DATAMEAN',DATAMEAN,hdr0)
if tag_exist(hdr0,'DATATOT') then hdr0.DATATOT=DATATOT else hdr0=create_struct('DATATOT',DATATOT,hdr0)
if tag_exist(hdr0,'DATASIG') then hdr0.DATASIG=DATASIG else hdr0=create_struct('DATASIG',DATASIG,hdr0)
if tag_exist(hdr0,'DATASKEW') then hdr0.DATASKEW=DATASKEW else hdr0=create_struct('DATASKEW',DATASKEW,hdr0)
if tag_exist(hdr0,'DATAKURT') then hdr0.DATAKURT=DATAKURT else hdr0=create_struct('DATAKURT',DATAKURT,hdr0)
if tag_exist(hdr0,'DATAP01') then hdr0.DATAP01=DATAP01 else hdr0=create_struct('DATAP01',DATAP01,hdr0)
if tag_exist(hdr0,'DATAP10') then hdr0.DATAP10=DATAP10 else hdr0=create_struct('DATAP10',DATAP10,hdr0)
if tag_exist(hdr0,'DATAP25') then hdr0.DATAP25=DATAP25 else hdr0=create_struct('DATAP25',DATAP25,hdr0)
if tag_exist(hdr0,'DATAP75') then hdr0.DATAP75=DATAP75 else hdr0=create_struct('DATAP75',DATAP75,hdr0)
if tag_exist(hdr0,'DATAP90') then hdr0.DATAP90=DATAP90 else hdr0=create_struct('DATAP90',DATAP90,hdr0)
if tag_exist(hdr0,'DATAP95') then hdr0.DATAP95=DATAP95 else hdr0=create_struct('DATAP95',DATAP95,hdr0)
if tag_exist(hdr0,'DATAP98') then hdr0.DATAP98=DATAP98 else hdr0=create_struct('DATAP98',DATAP98,hdr0)
if tag_exist(hdr0,'DATAP99') then hdr0.DATAP99=DATAP99 else hdr0=create_struct('DATAP99',DATAP99,hdr0)
if tag_exist(hdr0,'QUALITY') then hdr0.QUALITY=QUALITY else hdr0=create_struct('QUALITY',QUALITY,hdr0)

outputHdr=hdr0

status=1
;stop
end
