pro lst_prep, ihdr, iimg, ohdr, oimg, $
               bkgimg_on = bkgimg_on, bkg_hdr = bkg_hdr, bkg_dat = bkg_dat, $
               radcalib_on = radcalib_on, $
               fix_missing_off = fix_missing_off,  $             
               despike_on = despike_on, la_cosmic = la_cosmic, $
               reg_off = reg_off, $
               outsize=outsize, $
               do_write_fits = do_write_fits, compress=compress, outdir = outdir, $
               status = status, quiet = quiet, _EXTRA=ex

  ;+
; NAME: LST_PREP
;
; PROJECT: ASO-S/LST
;
; PURPOSE: optional preprocessing steps of LST level 1 data including:
;          background subtraction (usually for full-disk images),
;          radiometric calibration, 
;          fixing missing/bad pixels, 
;          fixing cosmic-ray/spike pixels, 
;          image registration (rotation, translation, scaling) 
; 
;          By default, 'lst_prep' fixes missing/bad pixels and does image registration.                                
;          If we want to do more, e.g., background substraction, radiometric calibration, despike, 
;          we need to set /bkgimg_on, /radcalib_on, /despike_on, respectively.
;
; CALL: wst_prep.pro, sdi_prep.pro
;      
; INPUTS: 
;   ihdr, iimg - header and image of level 1 data
; OUTPUTS:
;   ohdr, oimg - header and image of output data        
; KEYWORDS:       
;   /bkgimg_on - perform background subtraction, 
;                please use the keyword together with bkg_hdr & bkg_dat 
;   bkg_hdr & bkg_dat - if set /bkgimg_on, then please provide 
;            the header and data array for background (for full-disk observations)               
;   /radcalib_on - perform radiometric calibration     
;   /fix_missing_off - no fixing of the missing and bad pixels
;   /reg_off - no image registration
;   /despike_on - if set, then despike image (remove the cosmic ray hits) 
;   /la_cosmic - if set, then use la_cosmic method to despike image, 
;                please use together with /despike_on
;                note it takes about 30s to process an image of the size 4608*4608
;                approximately, the processing time is linearly proportional to the size of an image array
;   outsize - size of output data (only for full-disk images). If not set, it is the default size 4608
;   /do_write_fits - write a fits file 
;   /compress - write a fits.gz file   
;   outdir -  a folder for saving the fits file 
;   status - 0 for undone, 1 for successful, -1 for failed    
;   /quiet -  suppress output information                     
;                   
; CALLING SEQUENCE:
;
; ;use 'read_lst' to read the LST data before running 'lst_prep' 
; read_lst, fn, ihdr, iimg
;
; some examples for processing SDI data
; case 1: fixing missing/bad pixels, image registration
;  lst_prep, ihdr, iimg, ohdr, oimg
; case 2: fixing missing/bad pixels, despike, image registration
;  lst_prep, ihdr, iimg, ohdr, oimg, /despike_on
; case 3: fixing missing/bad pixels, despike, no image registration 
;  lst_prep, ihdr, iimg, ohdr, oimg, /despike_on, /reg_off
; case 3: background subtraction, fixing missing/bad pixels, despike, image registration  
;  lst_prep, ihdr, iimg, ohdr, oimg, /bkgimg_on, bkg_hdr=bkg_hdr, bkg_dat=bkg_dat, /despike_on  
; case 4: fixing missing/bad pixels, despike, writing compressed fits to outdir, outsize=2304 (half size)
;  lst_prep, ihdr, iimg, ohdr, oimg, /despike_on, /reg_off, outsize=2304, /do_write_fits, /compress, outdir=outdir 
; case 5: for cutout images: fixing missing/bad pixels, despike with the slower but maybe better la_cosmic method
;  lst_prep, ihdr, iimg, ohdr, oimg, /despike_on, /la_cosmic, /reg_off
;
; some examples for processing WST data 
; case 1: fixing missing/bad pixels, image registration
;  lst_prep, ihdr, iimg, ohdr, oimg
; case 2: radiometric calibration, fixing missing/bad pixels, image registration
;  lst_prep, ihdr, iimg, ohdr, oimg, /radcalib_on
; case 3: radiometric calibration, fixing missing/bad pixels, despike, image registration, 
;         writing compressed fits.gz to outdir  
;  lst_prep, ihdr, iimg, ohdr, oimg, /radcalib_on, /despike_on, /do_write_fits, /compress, outdir=outdir   
; case 4: fixing missing/bad pixels, despike, no image registration, outsize=4096 (same as HMI)
;  lst_prep, ihdr, iimg, ohdr, oimg, /despike_on, /reg_off, outsize=4096

; HISTORY: 
;        2022-07-12 Li Feng v0.9 
;        2023-04-02 Li Feng v1.0
;        2023-04-09 Li Feng change 'bkgimg_off', 'radcalib_off' to  'bkgimg_on', 'radcalib_on'            
   
    ;--Create structure to send keywords to subroutines
   IF datatype(ex) EQ 'UND' THEN ex = create_struct('blank',-1)

    ;--Pass QUIET and other keywords to subroutines
   IF keyword_set(QUIET) THEN ex = create_struct('quiet',1,ex)
    
   IF keyword_set(BKGIMG_ON) AND ~tag_exist(ex,'bkgimg_on') THEN $
      ex = create_struct('bkgimg_on',1,ex)
   IF keyword_set(RADCALIB_ON) AND ~tag_exist(ex,'radcalib_on') THEN $
      ex = create_struct('radcalib_on',1,ex)
   IF keyword_set(FIX_MISSING_OFF) AND ~tag_exist(ex,'fix_missing_off') THEN $
      ex = create_struct('fix_missing_off',1,ex)
   IF keyword_set(REG_OFF) AND ~tag_exist(ex,'reg_off') THEN $
      ex = create_struct('reg_off',1,ex)   
   IF keyword_set(DESPIKE_ON) AND ~tag_exist(ex,'despike_on') THEN $
      ex = create_struct('despike_on',1,ex)
   IF keyword_set(LA_COSMIC) AND ~tag_exist(ex,'la_cosmic') THEN $
      ex = create_struct('la_cosmic',1,ex)   
   IF keyword_set(DO_WRITE_FITS) AND ~tag_exist(ex,'do_write_fits') THEN $
      ex = create_struct('do_write_fits',1,ex)   
   IF keyword_set(COMPRESS) AND ~tag_exist(ex,'compress') THEN $
      ex = create_struct('compress',1,ex)          

    CASE ihdr[0].camera OF 
        'SDI': BEGIN
            SDI_PREP, ihdr, iimg, ohdr, oimg, bkg_hdr = bkg_hdr, bkg_dat = bkg_dat, $    
              outsize = outsize, outdir = outdir, status = status, _EXTRA=ex  
            END  
        'WST': BEGIN
            WST_PREP, ihdr, iimg, ohdr, oimg, $    
              outsize = outsize, outdir = outdir, status = status, _EXTRA=ex    
            END  
        ELSE: BEGIN
            print, 'invalid camera name...' 
            print, 'should be SDI, WST' 
        END
    ENDCASE

end
