
pro lst_radcalib, para1, para2, para3, para4, $
                  wst = wst, sdi = sdi, $
                  sciuv = sciuv, sciwl = sciwl, $
                  cfactor = cfactor, status = status
;+
; NAME: LST_RADCALIB
;
; PROJECT: ASO-S/LST
;
; PURPOSE: To convert LST level-1 imaging data from count rate 
;          (DN/s) to intensity (erg/s/cm^2/sr or MSB).
;
; CALL: read_lst.pro, lst_gen_hdr_cal.pro
;
; INPUTS: para1 - level 1 fits head, must be a structure type
;         para2 - 2D image array, in units of DN
;         [or para1 - level1 fits filename]                     
;                   
; OUTPUTS: para3 - fits head for outputs
;          para4 - 2D image array, in a unit of erg/s/cm^2/sr for WST, SDI, and SCIUV
;                  or MSB for SCIWL (calibrated intensity)
;          [or para2 & para3 for ohdr & oimg, if para1 is fits filename]
;          cfactor - calibration factor used
;          status - -1 for failed, 1 for successful         
;
; KEYWORDS: /wst - to calibrate the WST 360 nm image data
;           /sdi - to calibrate the SDI Lyman-alpha image data
;           /sciuv - to calibrate the SCI UV (Lyman-alpha) image data
;           /sciwl - to calibrate the SCI WL (700 nm) image data
;
; CALLING SEQUENCE: lst_radcalib, ihdr, idata, ohdr, odata, /wst[, /sdi, /sciuv, /sciwl]
;                   lst_radcalib, ifitsfilename, ohdr, odata, /wst[, /sdi, /sciuv, /sciwl]
;
; HISTORY: v0.5, 2020-Jun-01, written by ly (yingli@pmo.ac.cn)
;          v0.6, 2020-Jul-05, add lst_get_radcalib to get the calibration factor
;          v0.8, 2020-Nov-15, consider the exposure time
;          v0.9, 2021-March-21, delete keyword /sciwl (by ly)
;                2021-May-24, re-add keyword /sciwl (by ly)
;                2021-August-09, consider bad pixels -32767 etc. (by ly)
;                2021-August-15, consider attenuators for WST (by ly)
;                2021-August-27, add ihdr input & cfoactor output (by ly)
;          v1.0, 2021-November-29, add ohdr input & improve keywords (by ly)
;                2022-April-15, update the corresponding keywords (by ly)
;                2022-April-17, add fits filename as input (by ly)
;                2022-August-01, update the units for SCIUV (by ly)
;                2022-September-01, use struct for input and output (by ly)
;                2023-March-24, remove calling lst_get_radcalib.pro (by ly)
;
;-

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;     PROCEDURE CALL AND INITIAL SET UP     ;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

  status = 0

  ;One of the required keywords is set
  if (n_elements(wst) eq 0) and $
     (n_elements(sdi) eq 0) and $
     (n_elements(sciuv) eq 0) and $
     (n_elements(sciwl) eq 0) then begin
     message, 'ERROR', /continue
     print, $
        '>>> One of the keywords /wst, /sdi, /sciuv, /sciwl ' + $
        'must be set to run the procedure.'
     print, $
        '>>> These decide which calibrate factor to be used.'
     status = -1
     return
  endif

  if keyword_set(wst) + keyword_set(sdi) + keyword_set(sciuv) + keyword_set(sciwl) $
     ge 2 then begin
     message, 'ERROR', /continue
     print, $
        '>>> Select only one of the /wst, /sdi, /sciuv, /sciwl keywords'
     status = -1
     return
  endif

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;      INPUT TYPE & BAD PIXELS     ;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

  ;Some error messages to ensure that the correct inputs are provided
  case n_params() of
     3: begin
          level1fits = para1 
          read_lst, level1fits, ihdr, idata
        end
     4: begin
          ihdr = para1
          idata = para2
        end
     else: begin
           message, 'Error, Syntax: lst_radcalib, ihdr, idata, ohdr, odata ' $
                     + 'or Syntax: lst_radcalib, fitsfilename, ohdr, odata ', /continue
           status = -1
           return
           end
   endcase

  case datatype(ihdr) of
       'STR': begin
              hdr_tc = lst_fitshead2struct(ihdr)
              end
       'STC': begin
              hdr_tc = ihdr
              end
        else: begin
              message, 'Error, Syntax: lst_radcalib, ihdr, idata, ohdr, odata ' $
                       + 'or Syntax: lst_radcalib, fitsfilename, ohdr, odata ', /continue
              status = -1
              return
              end
  endcase

  bad1 = where(idata eq -32768, count1)
  bad2 = where(idata eq -32767, count2)

  idata = idata/hdr_tc.EXPTIME ;nomarlize the exposure time for radiometric calibration

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;     DEFINE SOME CONSTANTS & VARIABLES    ;;;;;;;;;;;;;;;;  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

  solid_angle = (hdr_tc.CDELT1*720.0*hdr_tc.CDELT2*720.0)/((1.5e8)^(2.0))

  ;calibration factors
  ;cf_wst = lst_get_radcalib(/wst) ; unit: erg/DN/cm^2
  ;cf_sdi = lst_get_radcalib(/sdi) ; unit: erg/DN/cm^2
  ;cf_sciuv = lst_get_radcalib(/sciuv) ; unit: erg/DN/cm^2
  ;cf_sciwl = lst_get_radcalib(/sciwl) ; unit: MSB/DN

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;     CONVERT THE COUNT RATE TO RADIANCE    ;;;;;;;;;;;;;;;;  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  
  ;wst data
  if keyword_set(wst) or hdr_tc.CAMERA eq 'WST' then begin
     cf_wst = 1.18503e-09 ;attn = 1 based on the data during 2023-01to03 (2.5 months)
    ;---
     if hdr_tc.IMG_ATTN eq 1 then attnc = 1.0 ;0.0053
     if hdr_tc.IMG_ATTN eq 2 then attnc = 0.5 ;0.0029
     if hdr_tc.IMG_ATTN ne 1 and hdr_tc.IMG_ATTN ne 2 then begin $
        message, 'ERROR: >>> no WST attenuator found!', /continue
        status = -1
        return
     endif
    ;---
     odata = idata*cf_wst/attnc/solid_angle
     if count1 gt 0 then odata[bad1] = -32768
     if count2 gt 0 then odata[bad2] = -32767
     cfactor = cf_wst
     odsatval = hdr_tc.DSATVAL/hdr_tc.EXPTIME*cf_wst/attnc/solid_angle
    ;---
     hdr_tc = add_tag(hdr_tc, cfactor, 'RAD_FACT')
     hdr_tc = add_tag(hdr_tc, 'erg/DN/cm^2', 'RAD_UNIT')
     hdr_tc = REP_TAG_VALUE(hdr_tc, 'erg/s/cm^2/sr', 'BUNIT')
     hdr_tc = REP_TAG_VALUE(hdr_tc, odsatval, 'DSATVAL')
    ;---
     status = 1
  endif

  ;sdi data
  if keyword_set(sdi) or hdr_tc.CAMERA eq 'SDI' then begin
     cf_sdi = 5.12900e-09 ;combine & refer to EUVS on 2022-10-26
     odata = idata*cf_sdi/solid_angle
     if count1 gt 0 then odata[bad1] = -32768
     if count2 gt 0 then odata[bad2] = -32767
     cfactor = cf_sdi
     odsatval = hdr_tc.DSATVAL/hdr_tc.EXPTIME*cf_sdi/solid_angle
    ;---
     hdr_tc = add_tag(hdr_tc, cfactor, 'RAD_FACT')
     hdr_tc = add_tag(hdr_tc, 'erg/DN/cm^2', 'RAD_UNIT')
     hdr_tc = REP_TAG_VALUE(hdr_tc, 'erg/s/cm^2/sr', 'BUNIT')
     hdr_tc = REP_TAG_VALUE(hdr_tc, odsatval, 'DSATVAL')
    ;---
     status = 1
  endif

  ;sciuv data
  if keyword_set(sciuv) or hdr_tc.CAMERA eq 'SCIUV' then begin
     cf_sciuv = 6.60000e-08
     odata = idata*cf_sciuv/solid_angle
     if count1 gt 0 then odata[bad1] = -32768
     if count2 gt 0 then odata[bad2] = -32767
     cfactor = cf_sciuv
     odsatval = hdr_tc.DSATVAL/hdr_tc.EXPTIME*cf_sciuv/solid_angle
    ;---
     hdr_tc = add_tag(hdr_tc, cfactor, 'RAD_FACT')
     hdr_tc = add_tag(hdr_tc, 'erg/DN/cm^2', 'RAD_UNIT')
     hdr_tc = REP_TAG_VALUE(hdr_tc, 'erg/s/cm^2/sr', 'BUNIT')
     hdr_tc = REP_TAG_VALUE(hdr_tc, odsatval, 'DSATVAL')
    ;---
     status = 1
  endif 

  ;sciwl data
  if keyword_set(sciwl) or hdr_tc.CAMERA eq 'SCIWL' then begin
     cf_sciwl = 6.57800e-11
     odata = idata*cf_sciwl
     if count1 gt 0 then odata[bad1] = -32768
     if count2 gt 0 then odata[bad2] = -32767
     cfactor = cf_sciwl
     odsatval = hdr_tc.DSATVAL/hdr_tc.EXPTIME*cf_sciwl
    ;---
     hdr_tc = add_tag(hdr_tc, cfactor, 'RAD_FACT')
     hdr_tc = add_tag(hdr_tc, 'MSB/DN', 'RAD_UNIT')
     hdr_tc = REP_TAG_VALUE(hdr_tc, 'MSB', 'BUNIT')
     hdr_tc = REP_TAG_VALUE(hdr_tc, odsatval, 'DSATVAL')
    ;---
     status = 1
  endif


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;     UPDATE RELATED KEYWORDS    ;;;;;;;;;;;;;;;;;;;;;;  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

  ;lst_gen_hdr_cal, hdr_tc, odata, outputHdr = hdr_t
  ohdr = hdr_tc

  if n_params() eq 4 then begin
      para3 = ohdr
      para4 = odata
  endif else begin
      para2 = ohdr
      para3 = odata
   endelse


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  
end
