;+
; NAME : lst_reg.pro (procedure)
; PURPOSE :
;   Register any image (rotate, translate, scale) to arbitrary reference values of scale, roll, and image center x and y
; PROJECT :
;   ASO-S / LST
; CATEGORY :
;   data converstion
; INPUTS :
; Case 1: References index structure and data 
;   ipara: input parameter of the image
;   idata: input data of image
; Case 2:  References FITS file name
;  ipara: the name of the fitsfile
; OUTPUTS :
;   opara: output parameter of the image
;   odata: output data of image
; KEYWORD PARAMETERS :
;   STATUS - Indicates whether the program is executing correctly 
;   INTERP - If set, use bilinear interpolation
;   CUBIC - If set, use cubic convolution interpolation ith the specified value as the interpolation parameter.
;              Cubic interpolation with this parameter equal -0.5 is the default.
;   MISSING - The data value to substitute for pixels in the output image that map outside the input image.
;   scale_ref - Image scaling reference value
;   roll_ref -  Image rotation angle reference value
;   crpix1_ref - Reference value in X-direction of image center
;   crpix2_ref - Reference value in Y-direction of image center
;   cutout -  when the image is cutout from full  disk data


; EXAMPLES :
;   
; MODIFICATION HISTORY :
;   Yu Huang, 2021/04/01 write
;   Yu Huang, 2021/08/10
;   Yu Huang, 2023/04/04 
;------------------------------------------------------------------------
;-
pro lst_reg, ipara, idata, opara, odata, status=status, cutout=cutout,$
   interp = interp, CUBIC=cubic, MISSING = missing,$
   scale_ref=scale_ref, roll_ref=roll_ref, crpix1_ref=crpix1_ref, crpix2_ref=crpix2_ref

prognam = 'lst_reg.pro'
progver = 'V1.0' ; 2021-02-03
 if exist(status) then status = 0
 if(n_params() ge 5) then begin
   message,'Error, Syntax : lst_reg, ipara, idata, opara, odata'
   return
 endif else begin 
  if (n_params() eq 3) then begin
  head_key_str = headfits(ipara)
  head_key = lst_fitshead2struct(head_key_str)
  fits2map,ipara,map
  idata=map.data
  ipara=head_key
  
  endif
 endelse
   
   if datatype(ipara) eq 'STR' then begin
     head_key=lst_fitshead2struct(ipara)
     opara=head_key
   endif else begin
     opara=ipara
     head_key=ipara
   endelse 
   

; Define default ROT interpolation and fill parameters
  if ((not exist(interp)) and (not exist(cubic))) then cubic = -0.5
  if not exist(missing) then missing = min(idata)

  if n_params() eq 0 then begin
    print, ' Program version number = ' + strtrim(progver,2)
    return
  endif

if head_key.cdelt1 eq 0 then head_key.cdelt1=1
if head_key.cdelt2 eq 0 then head_key.cdelt2=1
if not exist(roll_ref) then roll_ref=0

if keyword_set(cutout) then begin
  if not exist(roll_ref) then roll_ref=0
  if not exist(scale_ref) then scale_ref=head_key.cdelt1
   
  if exist(crpix1_ref) then x0 = crpix1_ref-1 else x0 = (head_key.naxis1+1.)/2-1
  if exist(crpix2_ref) then y0 = crpix2_ref-1 else y0 = (head_key.naxis2+1.)/2-1
endif else begin
  if not exist(roll_ref)  then roll_ref  = 0   ; default roll 0 degress
  if not exist(scale_ref) then scale_ref = head_key.cdelt1
  
  if exist(crpix1_ref) then x0 = crpix1_ref-1 else x0 = head_key.crpix1-1
  if exist(crpix2_ref) then y0 = crpix2_ref-1 else y0 = head_key.crpix2-1
endelse
   
  

  
delta_roll=roll_ref-head_key.crota2
scale_fac=head_key.cdelt1/scale_ref


  odata = rot(idata, delta_roll, scale_fac, x0, y0, interp=interp, cubic=cubic, missing=missing)

 opara=REP_TAG_VALUE(opara,roll_ref,'crota2')
 if keyword_set(cutout) then begin
  
  
  crp1=x0+cos(delta_roll*!dtor)*(head_key.crpix1-x0)+sin(delta_roll*!dtor)*(head_key.crpix2-y0)
  crp2=y0-sin(delta_roll*!dtor)*(head_key.crpix1-x0)+cos(delta_roll*!dtor)*(head_key.crpix2-y0)
 
  opara=REP_TAG_VALUE(opara,crp1,'crpix1')
  opara=REP_TAG_VALUE(opara,crp2,'crpix2')
  opara=REP_TAG_VALUE(opara,0d,'xcen')
  opara=REP_TAG_VALUE(opara,0d,'ycen')
  opara=REP_TAG_VALUE(opara,0d,'rcen')
 
 endif else begin
  opara=REP_TAG_VALUE(opara,(head_key.naxis1+1.)/2,'crpix1')
  opara=REP_TAG_VALUE(opara,(head_key.naxis2+1.)/2,'crpix2') 
  opara=REP_TAG_VALUE(opara,0d,'xcen')
  opara=REP_TAG_VALUE(opara,0d,'ycen')
  opara=REP_TAG_VALUE(opara,0d,'rcen') 

 endelse
 if (n_params() eq 3) then begin
  idata=opara
  opara=odata
  endif
  if datatype(ipara) eq 'STR' then begin
    opara=struct2fitshead(opara)
  endif
  if exist(status) then status = 1
end
