pro read_lst, lstfiles, index, data, nodata=nodata, fitshead=fitshead
  ; =========================================================================
  ;+
  ; PROJECT:
  ;       ASO-S/LST
  ; NAME:
  ;       READ_LST
  ; PURPOSE:
  ;       Read LST FITS file(s) into 2D or 3D array
  ; INPUTS:
  ;       LSTFILES - LST FITS file(s) name to read
  ; KEYWORDS:
  ;       /NODATA  - Read in header(s) only, not data.
  ;       /FITSHEAD  - If set, output string index
  ; OUTPUTS:
  ;       INDEX   - Output the data header(s)
  ;       DATA    - 2D or 3D array of LST image(s)
  ; EXAMPLES:
  ;       IDL> read_lst, lstfiles, index, data [,/nodata] [,/fitshead]
  ; NOTES:
  ;       The input files must have the same size of data
  ;       if multiple data are read
  ; HISTORY:  2021-October-12; Written by lijw (njlijw@pmo.ac.cn)
  ; HISTORY:  2022-April-07  ; Readfits.pro is replaced with mreadfits.pro
  ; HISTORY:  2022-April-12  ; Default output structure index, and add keyword "fitshead"
  ; HISTORY:  2022-June-28   ; After modification, readfits version 2013 can be used correctly
  ; HISTORY:  2022-July-15   ; using lst_fitshead2struct instead of fitshead2struct
  ; HISTORY:  2022-Sep-24    ; Added to check whether the read FITS data is correct or complete
  ;progver = 'v2023-Feb-08'   ; Force the header keyword 'bscale' to be set to 1.0 when the data is read,
  ;                           ; if only reading the header does not change the assignment of that keyword.
  ; =========================================================================

  ; === CHECK THE OUTPUT PARAMETERS =========================================
  lstf = (n_params() eq 3) and (keyword_set(nodata) eq 0)
  lstfn = (keyword_set(nodata) or (n_params() eq 2))
  prognam='READ_LST'
  ; === CHECK THE NUMBER OF PARAMETERS ======================================
  if ((n_params() lt 2) or (n_params() gt 3)) then begin
    box_message, 'READ_LST: Incorrect number of parameters.'
    return
  endif

  ; === CHECK THE LST FILES EXIST OR NOT =====================================
  nfe   = file_exist(lstfiles)
  nfne  = where(nfe eq 0)
  nfe   = where(temporary(nfe) eq 1, nf)

  if (nf ne n_elements(lstfiles)) then begin
    box_message, ['READ_LST: Cannot find all the LST files ' $
      + 'Read the found files and continue.']
  endif

  ; === IF CANNOT FIND ANY FILES THEN RETURN =================================
  if (nf eq 0) then begin
    box_message,  ['READ_LST: Cannot find any of the LST files']
    return
  endif

  ; === READING THE FILES ===================================================
  if lstf then begin
    fits_open,lstfiles[nfe[0]],fcb
    fits_close,fcb
    
    if (1 - tag_exist(fcb,'filename')) then begin
      box_message, 'The input fits file contains some errors. Please check it...',/info
      status=-1
      return
    endif 
    
    if (fcb.nextend eq 0) then begin
       ;;;;;check the data size and the fitsheader is the same
        if nf gt 1 then begin       
             nx=strarr(nf) & ny=strarr(nf)
             size_different = bytarr(nf)
             hnum=strarr(nf)
           for j=0,nf-1 do begin
             head = headfits(lstfiles[nfe[j]],ext=0,silent=quiet)   ; pretty fast header-only read
             hnumd=n_elements(head)
             hnum[j]=hnumd
             nx[j] = gt_tagval(head, /naxis1)
             ny[j] = gt_tagval(head, /naxis2)
           endfor

           for jj = 0, nf-1 do begin
              size_different[jj] = (nx[jj] ne max(nx)) or  (ny[jj] ne max(ny))
           endfor

           if (total(size_different) ne 0) then begin
              box_message,                                $
               prognam+': Files must have the same size data.'
             qabort = 1B
              status=-1
             return
            endif
         endif
    ;;;;;;;;;;;;;read the data (2-D OR 3-D)
         data = readfits(lstfiles[nfe[0]],ext=0,header)
         header = headfits(lstfiles[nfe[0]],ext=0)
         fxaddpar, header, 'bscale', 1

         hnumx=n_elements(header)
         index=strarr(max(hnumx)*2,nf)
         index(0:n_elements(header)-1,0)=header 
                      
         if nf eq 1 then begin
              if keyword_set(fitshead) then begin
                   index=index(0:hnumx-1,0:nf-1)
              endif else begin
                  index=[lst_fitshead2struct(index, ncomment=20, nhistory=20)]
              endelse
         endif
         
         if nf gt 1 then begin
              for iii=1,nf-1 do begin
                  dtemp=readfits(lstfiles[nfe[iii]],ext=0,header)
                  header=headfits(lstfiles[nfe[iii]],ext=0)
                  fxaddpar, header, 'bscale', 1
                  data=[[[data]],[[dtemp]]]
                  index(0:n_elements(header)-1,iii)=header
                  hnumx=max([hnumx,n_elements(header)])
              endfor
                   index=index(0:hnumx-1,0:nf-1)
              if keyword_set(fitshead) then begin
                    index=index
              endif else begin
                      tindex=[lst_fitshead2struct(index[*,0], ncomment=20, nhistory=20)]
                  for ii=1, nf-1 do begin
                      tindex=[tindex,lst_fitshead2struct(index[*,ii], ncomment=20, nhistory=20)]
                  endfor
                      index=tindex
               endelse
             
          endif
        return
    endif
    
    ; === JUST READ SINGLE SPIKE FITS FILE ============================================ 
    if (fcb.nextend eq 1) then begin
         if (nf eq 1) then begin
             data = readfits(lstfiles[nfe[0]],ext=1,header)  
             header=headfits(lstfiles[nfe[0]],ext=1)  
             fxaddpar, header, 'bscale', 1    
            if keyword_set(fitshead) then begin
                  index=header
            endif else begin
                  index=lst_fitshead2struct(header, ncomment=20, nhistory=20)
            endelse
        endif
        if (nf gt 1) then box_message,                       $
             ['READ_LST: Read only one Spike fit file at a time. Aborting.']              
        return
    endif
         
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    if (fcb.nextend gt 1)then begin
        box_message,                                $
        prognam+': Please check the input data, the data includes more than one HDU.'         
        return
    endif
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
         
         
         
  endif

  ; === USING THE KEYWORD NODATA  ===========================================
  if lstfn then begin
     fits_open,lstfiles[nfe[0]],fcb
     fits_close,fcb
     
     if (1 - tag_exist(fcb,'filename')) then begin
       box_message, 'The input fits file contains some errors. Please check it...',/info
       status=-1
       return
     endif
     
     if (fcb.nextend eq 0) then begin
         mreadfits, lstfiles[nfe] ,index

         if (nf eq 1) and keyword_set(fitshead) then begin
               index=headfits(lstfiles[nfe[0]],ext=0)
         endif

         if (nf gt 1) and keyword_set(fitshead) then begin 
         ; === READ THE HEADER OF FIRST ONE ========================================
                   header = headfits(lstfiles[nfe[0]],ext=0)
                   hnumx=n_elements(header)
                   index=strarr(max(hnumx)*2,nf)
                   index(0:n_elements(header)-1,0)=header
         ; === READ THE OTHER HEADERS AND NUMBER ===================================
             for iii=1,nf-1 do begin
                    header=headfits(lstfiles[nfe[iii]],ext=0)
                   index(0:n_elements(header)-1,iii)=header
                   hnumx=max([hnumx,n_elements(header)])
              endfor
                   index=index(0:hnumx-1,0:nf-1)
          endif

     endif

    ; === JUST READ SINGLE SPIKE FITSHEAD  ========================================= 
     if (fcb.nextend eq 1) then begin
         if (nf eq 1) then begin        
             header=headfits(lstfiles[nfe[0]],ext=1)
             if keyword_set(fitshead) then begin
                   index=header
             endif else begin
                   index=lst_fitshead2struct(header, ncomment=20, nhistory=20)
             endelse
         endif       
       
         if (nf gt 1) then begin
          ; === READ THE HEADER OF FIRST ONE ========================================
                 header = headfits(lstfiles[nfe[0]],ext=1)
                 hnumx=n_elements(header)
                 index=strarr(max(hnumx)*2,nf)
                 index(0:n_elements(header)-1,0)=header
         ; === READ THE OTHER HEADERS AND NUMBER ===================================           
           for iii=1,nf-1 do begin
                 header=headfits(lstfiles[nfe[iii]],ext=1)
                 index(0:n_elements(header)-1,iii)=header
                 hnumx=max([hnumx,n_elements(header)])
           endfor
         
              index=index(0:hnumx-1,0:nf-1)
         
           if keyword_set(fitshead) then begin
                  index=index
           endif else begin          
                     tindex=lst_fitshead2struct(index[*,0], ncomment=20, nhistory=20)
                for ii=1, nf-1 do begin
                     tindex=[tindex,lst_fitshead2struct(index[*,ii], ncomment=20, nhistory=20)]
                endfor
               index=tindex
           endelse
         endif     
       
     endif
      
     ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
     if (fcb.nextend gt 1)then begin
       box_message,                                $
         prognam+': Please check the input data, the data includes more than one HDU.'        
       return
     endif
     ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
  endif

  ; =========================================================================
  return
end
