;+
; NAME : 
;   sdi_ind_spike.pro (procedure)
; PURPOSE :
;   Get positions, values and the quantity of spikes from LST / SDI level 1.0 data,
;   and update the 'NSPIKES' in inputHdr.
; PROJECT :
;   ASO-S / LST
; CATEGORY :
;   Data processing
; CALLING SEQUENCE :
;   sdi_ind_spike,inputHdr,inputData,outputHdr=outputHdr,outputData=outputData,spkPos=spkPos,spkValue=spkValue,
;   nSpks=nSpks,sigmas=sigmas,sbin=sbin,minthr=minthr,la_cosmic=la_cosmic,niter=niter
; INPUTS :
;   inputHdr = the file header of the inputData
;   inputData = a data array with 2 dimensions (2D)
; OPTIONAL INPUT PARAMETERS :
;   None
; OUTPUTS :
;   None
; OPTIONAL OUTPUT PARAMETERS :
;   outputHdr: the updated fits head
;   spkPos: the positions of spikes
;   spkValue: the values of spikes
;   nSpks: the quantity of spikes
; KEYWORD PARAMETERS :
;   sigmas: the threshold for cosmic ray detection in default method
;   sbin: the kernel size for median filter in default method
;   minthr: the minimum relative detection threshold in default method
;   la_cosmic: using the ind_la_cosmic method to index spikes
;   niter: how many passes to take in ind_la_cosmic method
;   status: a variable to indicate whether the program are successful. 1 for successful, -1 for failed.
; COMMON BLOCKS :   
;   None
; SIDE EFFECTS :
;   None
; RESTRICTIONS :
;   Usable for arrays with 2D and its header.
; PROCEDURE :
;   ind_la_cosmic.pro
; EXAMPLE :
;   sdi_ind_spike,inputHdr,inputData,outputHdr=outputHdr,outputData=outputData,spkValue=spkValue
;   sdi_ind_spike,inputHdr,inputData,nSpks=nSpks,spkPos=spkPos,sigmas=4.,sbin=9,/la_cosmic
; MODIFICATION HISTORY :
;   2020/10/25 De-Chao Song (dcsong@pmo.ac.cn)   
;   2020/11/1 De-Chao Song (dcsong@pmo.ac.cn)   
;   2020/12/30 De-Chao Song (dcsong@pmo.ac.cn)   
;   2021/1/14 De-Chao Song (dcsong@pmo.ac.cn)   
;   2021/3/30 De-Chao Song (dcsong@pmo.ac.cn) add another medthod (ind_la_cosmic) to index the spikes
;   2021/6/4 De-Chao Song (dcsong@pmo.ac.cn)   add status keyword
;   2021/6/16 De-Chao Song (dcsong@pmo.ac.cn)   consider the case for no spikes found

;------------------------------------------------------------------------
;-
pro sdi_ind_spike,inputHdr,inputData,outputHdr=outputHdr,outputData=outputData,spkPos=spkPos,spkValue=spkValue,$
nSpks=nSpks,sigmas=sigmas,sbin=sbin,minthr=minthr,la_cosmic=la_cosmic,niter=niter,status=status,_EXTRA=EXTRA_KEYWORDS

inputHdr0=inputHdr
;help,inputhdr0
IF datatype(inputHdr0) NE 'STC' then inputHdr0=lst_fitshead2struct(inputHdr0)

status=0
;Check the dimension of inputData.
dsize=size(inputData)
type=size(inputData,/type)
image=inputData
;;;;sdc 2023-03-29
ind_thre=20
valid=WHERE(image GE ind_thre,cnt,/null);,COMPLEMENT=invalid,NCOMPLEMENT=incnt)

IF valid EQ !null THEN BEGIN
  message,'No data great than or equal to zero, please check it.',/continue
  status=-1
  return
ENDIF

nx=dsize[1]
ny=dsize[2]

; IF(DATATYPE(inputHdr0) NE 'STR') THEN inputHdr0=struct2fitshead(inputHdr0,/allow_crota)
IF dsize[0] NE 2 THEN BEGIN
  message,'Dimension of inputData is not right. Please input a 2D array.',/continue
  status=-1
  return
ENDIF

IF NOT keyword_set(sigmas) THEN sigmas = 60 ; Threshold for cosmic ray detection
IF NOT keyword_set(sbin) THEN sbin = 11 ; Kernel size for median filter
IF NOT keyword_set(minthr) THEN minthr =0.5 ; Minimum relative detection threshold
IF NOT keyword_set(niter) THEN niter=4 ; The default iteration for the la_cosmic method


IF keyword_set(la_cosmic) THEN BEGIN
;;;the la_cosmic method
ind_la_cosmic,image,niter=niter,www=hit
ENDIF ELSE BEGIN 
;;;the fast method
IF (cnt GT 0L) THEN BEGIN
  ; imalim=FLTARR(nx,ny)
  ; imamed=FLTARR(nx,ny)
  imalim=make_array(nx,ny,value=ind_thre,/float)
  imamed=imalim

  IF cnt GE sbin THEN BEGIN
  imamed[valid]=MEDIAN(image[valid],sbin)  
  ENDIF ELSE BEGIN 
  message,'SBIN great than the width of valid data. Please check the data or minish the SBIN value.',/continue
  status=-1
  return
  ENDELSE

  imalim[valid]=imamed[valid]+$
         ( sigmas*SQRT( 2.7 + 0.2*imamed[valid] )>       $
           imamed[valid]*minthr )
  hit=WHERE(image GT imalim,/null)
ENDIF ELSE BEGIN 
  hit=!null
ENDELSE

ENDELSE


spkPos=hit
print,'++++++++++++++++++++++++++++'
IF hit EQ !null THEN BEGIN
nSpks=0
spkValue=!null
outputData=[0]
message,'No spikes found.',/continue

ENDIF ELSE BEGIN 
spkValue=inputData[spkPos]
nSpks=n_elements(spkPos)
outputData=[[spkPos],[spkValue]]
message,strtrim(string(nSpks),2)+' spikes found.',/continue

ENDELSE
print,'++++++++++++++++++++++++++++'


; fxaddpar,inputHdr0,'NSPIKES', nSpks, 'Number of cosmic ray affected pixel spikes in image'
if tag_exist(inputHdr0,'NSPIKES') then inputHdr0.NSPIKES=nSpks else inputHdr0=create_struct('NSPIKES',nSpks,inputHdr0)
;print,inputHdr0.NSPIKES

outputHdr=inputHdr0

status=1

end
