
pro sdi_prep, ihdr, iimg, ohdr, oimg, $ 
              bkgimg_on = bkgimg_on, $
              bkg_hdr = bkg_hdr, bkg_dat = bkg_dat, $
              radcalib_on = radcalib_on, $
              fix_missing_off = fix_missing_off, $             
              despike_on = despike_on, la_cosmic = la_cosmic, $             
              reg_off = reg_off, outsize = outsize, $
              do_write_fits = do_write_fits, compress = compress, $
              outdir = outdir, status = status, $
              quiet = quiet, _EXTRA=ex

;+
; NAME: SDI_PREP
;
; PROJECT: ASO-S/LST
;
; PURPOSE: To process the level-1 SDI data for different purposes
;          0. do background subtraction or not
;          1. do radiometric calibration or not
;          2. fix missing pixels or not
;          3. despike image or not
;          4. make registration (rotation, translation, scaling) or not
;
; CALL: lst_radcalib.pro, lst_reg.pro, lst_gen_hdr_cal.pro, disk_despike.pro, 
;       sdi_fix_missing.pro, lst_fitshead2struct.pro, lst_update_history.pro
;
; INPUTS: must be header & image array (2D) of SDI level-1 data                              
;                   
; OUTPUTS: header & image array (2D) for different purposes
;
; KEYWORDS: by default (no keywords set): do background subtraction, 
;           radiometric calibration, fix missing pixels, and 
;           make registration (rotation, translation, scaling)
;   /bkgimg_on - if set, then do background subtraction 
;   /bkg_hdr & /bkg_dat - if do background subtraction, then please provide 
;                         the header and data array for background (full-disk)
;   /radcalib_on - if set, then do radiometric calibration
;   /fix_missing_off - if set, then not fix missing pixels
;   /despike_on - if set, then despike image (remove the cosmos ray hits)
;   /la_cosmic - if set, then use la_cosmic method to despike image
;                        NOTE, this is very slow
;   /reg_off - if set, then not make registration
;   /outsize - if set, then apply to the oimg array (via congrid) & ohdr
;              note that outsize can be 4608, 2304, 1152, 576, 288, 144 etc.
;   /do_write_fits - if set, then write fits for output image data
;   /compress - if set, then compress the output fits
;   /outdir  - if set, then the output fits file will be written in outdir
;              default is the current directory
;   /status - 0 for undone, 1 for successful, -1 for failed
;   /quiet - if set, then quiet the info or message
;
; CALLING SEQUENCE: sdi_prep, ihdr, iimg, ohdr, oimg[, /bkgimg_on, /radcalib_on, $
;                             /fix_missing_off, /reg_off, /despike_on, $
;                             /do_write_fits]
;
; EXAMPLES:
;   case 1: by default (no keywords set): 
;        to fix missing pixels and make registration
;        IDL> sdi_prep, ihdr, iimg, ohdr, oimg
;   case 2: 
;        to do background subtraction, radiometric calibration, fix missing pixels,
;           make registration, despike image, i.e., do all the four proccesses
;        IDL> sdi_prep, ihdr, iimg, ohdr, oimg, /bkgimg_on, bkg_hdr = bkg_hdr, $
;                                   bkg_dat = bkg_dat, /radcalib_on, /despike_on
;   case 3: nothing to do
;        IDL> sdi_prep, ihdr, iimg, ohdr, oimg, /fix_missing_off, /reg_off
;
; HISTORY: v0.5, 2020-SEPTEMBER-01, written by ly (yingli@pmo.ac.cn)
;                2020-SEPTEMBER-25, added more keywords (by ly)
;          v0.8, 2021-October-11, update keywords (by ly)
;          v1.0, 2021-October-15, initial version (by ly)
;                2021-October-18, improve the logic flow (by ly)
;                2021-October-29, change a different logic way (by ly)
;                2021-December-28, add multi-inputs (by ly)
;                2022-April-08, call disk_despike rather than wst_despike (by ly)
;                2022-April-11, add /compress (by ly)
;                2022-May-27, add status keywords (by ly)
;                2022-Jul-15, call lst_fitshead2struct.pro (by ly)
;                2022-Aug-19, remove /spike_dir, add /la_cosmic (by ly)
;                2022-Aug-20, add /outsize for oimg & ohdr (by ly)
;                2022-Aug-23, change fltarr to dblarr for multiple inputs (by ly)
;                2022-Aug-29, call lst_update_history.pro for history (by ly)
;                2023-March-24, remove calling lst_get_radcalib.pro (by ly)
;                               add /bkgimg_on for background subtraction (by ly)
;                2023-April-02, add /outsize (by ly)
;                2023-April-04, add /bkg_hdr and /bkg_dat (by ly)
;
;-

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;     INITIAL SET UP     ;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

  status = 0

  if n_params() ne 4 then begin
     message, 'ERROR', /continue
     print, '>>> Syntax: sdi_prep, ihdr, iimg, ohdr, oimg'
     status = -1
     return
  endif

  ;--Create structure to send keywords to subroutines
  IF datatype(ex) EQ 'UND' THEN ex = create_struct('blank', -1)
  ;--Pass QUIET keyword to subroutines
  IF keyword_set(QUIET) THEN ex = create_struct('quiet', 1, ex)

  if keyword_set(outdir) then odir = outdir else odir = './'

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;     TRANSFORM THE DATA    ;;;;;;;;;;;;;;;;;;;;;;;  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

if datatype(ihdr) ne 'STR' and datatype(ihdr) ne 'STC' then begin 
   message, 'ERROR', /continue
   print, '>>> Syntax: ihdr should be a structure or a string (array).'
   status = -1
   return
endif

if datatype(ihdr) eq 'STR' then begin
   if (size(ihdr))[0] eq 1 then begin
      ni = 1
      hdr_in_tc = lst_fitshead2struct(ihdr)
   endif else begin
      ni = (size(ihdr))[2]
      hdr0 = lst_fitshead2struct(ihdr[*,0])
      hdr_in_tc = replicate(hdr0, ni)
      for i = 0, ni - 1 do begin
          hdr_in_tc[i] = lst_fitshead2struct(ihdr[*,i])
      endfor
   endelse
endif

if datatype(ihdr) eq 'STC' then begin
   ni = n_elements(ihdr)
   hdr_in_tc = ihdr
endif


for i = 0, ni - 1 do begin

  message, '=========> processing fits file ...', /info
  message, '=========> ' + hdr_in_tc[i].filename, /info

  if keyword_set(bkgimg_on) then begin
    ;---
     if ni eq 1 then begin
        hdr_in = hdr_in_tc
        img_in = iimg
     endif else begin
        hdr_in = hdr_in_tc[i]
        img_in = iimg[*,*,i]
     endelse
    ;---
     if ~keyword_set(quiet) then message, '=========> doing background subtraction ...', /info
     nsize = size(img_in)
     if nsize[1] eq 4608 and nsize[2] eq 4608 then begin
       hdr1n = hdr_in
       exptim = hdr1n.EXPTIME
       bad1 = where(img_in eq -32768, count1)
       bad2 = where(img_in eq -32767, count2)
       img1n = (img_in/exptim - bkg_dat)*exptim
       if count1 gt 0 then img1n[bad1] = -32768
       if count2 gt 0 then img1n[bad2] = -32767
       ;---
       hdr1n = REP_TAG_VALUE(hdr1n, bkg_hdr.FILENAME, 'BKG_NAME')
       hdr1n = lst_update_history(hdr1n, 'Background subtracted')
       ;---
       hdr_out = hdr1n
       img_out = img1n
     endif else begin
       hdr_out = hdr_in
       img_out = img_in
     endelse
    ;---
     if ~keyword_set(quiet) then message, '=========> background subtraction done ...', /info
  endif else begin
    ;---
     if ni eq 1 then begin
        hdr_out = hdr_in_tc
        img_out = iimg
     endif else begin
        hdr_out = hdr_in_tc[i]
        img_out = iimg[*,*,i]
     endelse
    ;---
  endelse

  if keyword_set(radcalib_on) then begin
    ;---
     hdr_in = hdr_out
     img_in = img_out
    ;---
     if ~keyword_set(quiet) then message, '>>>>>>>>>> doing radcalib ...', /info
     lst_radcalib, hdr_in, img_in, hdr_out, img_out, /sdi
     hdr_out = lst_update_history(hdr_out, 'Radiometric calibration done', /history)
     if ~keyword_set(quiet) then message, '=========> radcalib done ...', /info
    ;---
  endif

  if ~keyword_set(fix_missing_off) then begin
    ;---
     hdr_in = hdr_out
     img_in = img_out
    ;---
     if ~keyword_set(quiet) then message, '>>>>>>>>>> fixing missing ...', /info
     sdi_fix_missing, hdr_in, img_in, hdr_out, img_out
     hdr_out = lst_update_history(hdr_out, 'Missing pixels fixed', /history)
     if ~keyword_set(quiet) then message, '=========> missing fixing done ...', /info
    ;---
  endif

  if keyword_set(despike_on) then begin
    ;---
     hdr_in = hdr_out
     img_in = img_out
    ;---
     if ~keyword_set(quiet) then message, '>>>>>>>>>> doing despike ...', /info
     if keyword_set(la_cosmic) then $
        disk_despike, hdr_in, img_in, hdr_out, img_out, /la_cosmic $
     else disk_despike, hdr_in, img_in, hdr_out, img_out
     hdr_out = lst_update_history(hdr_out, 'Spikes removed', /history)
     if ~keyword_set(quiet) then message, '=========> despike done ...', /info
    ;---
  endif

  if ~keyword_set(reg_off) then begin
    ;---
     hdr_in = hdr_out
     img_in = img_out
    ;---
     if ~keyword_set(quiet) then message, '>>>>>>>>>> doing registration ...', /info
     nsize = size(img_in)
     if nsize[0] eq 2 and nsize[1] eq 4608 and nsize[2] eq 4608 then begin
        lst_reg, hdr_in, img_in, hdr_out, img_out
     endif else begin
        lst_reg, hdr_in, img_in, hdr_out, img_out, /cutout
     endelse
     hdr_out = lst_update_history(hdr_out, 'Registration done', /history)
     if ~keyword_set(quiet) then message, '=========> registration done ...', /info
    ;---
  endif

  if keyword_set(outsize) then begin
     img_out = congrid(img_out, outsize, outsize)
     outsize=float(outsize)
    if hdr_out.NAXIS1 eq 4608 and hdr_out.NAXIS2 eq 4608 then begin
       hdr_out.W1COL = 1
       hdr_out.W2COL = outsize
       hdr_out.W1ROW = 1
       hdr_out.W2ROW = outsize
    endif else begin
       hdr_out.W1COL = round(hdr_out.W1COL*outsize/hdr_out.NAXIS1)>1
       hdr_out.W2COL = hdr_out.W1COL+outsize-1
       hdr_out.W1ROW = round(hdr_out.W1ROW*outsize/hdr_out.NAXIS2)>1
       hdr_out.W2ROW = hdr_out.W1ROW+outsize-1
    endelse
    hdr_out.SUM_COL = round(hdr_out.NAXIS1/outsize)
    hdr_out.SUM_ROW = round(hdr_out.NAXIS2/outsize)
    hdr_out.CDELT1 = hdr_out.NAXIS1*hdr_out.CDELT1/outsize
    hdr_out.CDELT2 = hdr_out.NAXIS2*hdr_out.CDELT2/outsize
    hdr_out.CRVAL1=(hdr_out.CRPIX1-(hdr_out.NAXIS1+1)/2.)*outsize/hdr_out.NAXIS1
    hdr_out.CRVAL2=(hdr_out.CRPIX2-(hdr_out.NAXIS2+1)/2.)*outsize/hdr_out.NAXIS2 
    tem_crval1=(hdr_out.CRPIX1-(hdr_out.NAXIS1+1)/2.)*outsize/hdr_out.NAXIS1
    tem_crval2=(hdr_out.CRPIX2-(hdr_out.NAXIS2+1)/2.)*outsize/hdr_out.NAXIS2
    hdr_out.CRPIX1 = tem_crval1+(outsize+1)/2.
    hdr_out.CRPIX2 = tem_crval2+(outsize+1)/2.
    hdr_out.NAXIS1 = outsize
    hdr_out.NAXIS2 = outsize
    hdr_out.TOTVALS=long(outsize)*long(outsize)
    tmp = where(img_out gt 0 and img_out lt hdr_out.DSATVAL, n_DATAVALS)
    hdr_out.DATAVALS=n_DATAVALS
    hdr_out.MISSVALS = hdr_out.TOTVALS - hdr_out.DATAVALS
    hdr_out.PERCENTD = (float(hdr_out.DATAVALS)/hdr_out.TOTVALS)*100.0
    tmp=where(img_out eq 0., n_DATAZER)
    hdr_out.DATAZER=n_DATAZER
    tmp=where(img_out ge hdr_out.DSATVAL, n_datasat)
    hdr_out.DATASAT=n_datasat
 endif

  lst_gen_hdr_cal, hdr_out, img_out, outputHdr = hdr_out

 ;---
  if strmid(hdr_out.filename, 34, 3) eq 'cut' then $
     ofilename = strmid(hdr_out.filename, 0, 3) + $
                 strmid(hdr_out.filename, 9, 33) $
  else $
     ofilename = strmid(hdr_out.filename, 0, 3) + $
                 strmid(hdr_out.filename, 9, 29)
  hdr_out = REP_TAG_VALUE(hdr_out, ofilename, 'FILENAME')
  hdr_out = REP_TAG_VALUE(hdr_out, '-1', 'LVL_NUM')
  hdr_out = REP_TAG_VALUE(hdr_out, 1, 'BSCALE')
  hdr_out = REP_TAG_VALUE(hdr_out, 0, 'BZERO')
 ;---

  if keyword_set(do_write_fits) then begin
     if keyword_set(compress) then begin
        writefits, concat_dir(odir, ofilename), img_out, struct2fitshead(hdr_out), /compress
     endif else begin
        writefits, concat_dir(odir, ofilename), img_out, struct2fitshead(hdr_out)
     endelse
  endif

 ;---
  if ni eq 1 then begin 
     ohdr = hdr_out
     oimg = img_out
  endif else begin
     if i eq 0 then begin
        ohdr = replicate(hdr_out, ni)
        oimg = dblarr(hdr_out.naxis1, hdr_out.naxis2, ni)
        ohdr[0] = hdr_out
        oimg[*, *, 0] = img_out
     endif else begin
        ohdr[i] = hdr_out
        oimg[*, *, i] = img_out
     endelse
  endelse
 ;---

endfor 

status = 1

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  
end
