;+
; NAME : wst_fix_missing.pro (procedure)
; PURPOSE :
;   Fix bad or missing pixels of WST image and update the statistic in file header.
; PROJECT :
;   ASO-S / LST
; CATEGORY :
;   data processing / data analysis
; CALLING SEQUENCE :
;   wst_fix_missing,p1,p2,p3,p4,ni=ni,img_n=img_n,img_p=img_p,nMisses=nMisses,status=status
; INPUTS :
;   hdr_in = the file header of img_in (data array)
;   img_in = image to be fixed (fitsfile or a data array with 2 dimensions)
; OPTIONAL INPUT PARAMETERS :
;   img_n = the image observed immediately before img_in
;   img_p = the image observed immediately after img_in
; OUTPUTS :
;   img_out = the image fixed
;   hdr_out = the header updated
; OPTIONAL OUTPUT PARAMETERS :
;   nMisses = the quantity of missing pixels
;   status = whether the process has been executed
; KEYWORD PARAMETERS :
;   ni = the number of iterations
; COMMON BLOCKS :
;   None
; SIDE EFFECTS :
;   None
; RESTRICTIONS :
;   The input image should be a fitsfile or a 2D data array
; PROCEDURES :
;  read_lst
; EXAMPLES :
;   wst_fix_missing,(hdr1,img1)/fitsfile_lv1,hdr15,img15
; MODIFICATION HISTORY :
;   2021/02/02 Yuechun Song (ycsong@pmo.ac.cn)
;   2021/03/25 Yuechun Song
;   2021/04/01 Yuechun Song - Replaced readfits with read_lst
;   2021/05/14 Yuechun Song - Added status
;   2021/08/24 Yuechun Song - Simplified
;   2021/12/30 Yuechun Song - Added nMisses
;   2022/08/22 PingZhang - change missvals, datavals and percentd in fits header
;   2022/08/29 PingZhang - make corrections in function mb_detect
;   2022/09/06 PingZhang - change input header from string to structure
;------------------------------------------------------------------------
;-

function mb_detect,x
  mb1=where(x eq -32768,/null,count)
  if count eq 0 then begin
    y=where(x eq -32767)
  endif else begin
    mb2=where(x eq -32767,/null)
    y=setunion(mb1,mb2)
  endelse
  return,y
end

pro wst_fix_missing,p1,p2,p3,p4,ni=ni,img_n=img_n,img_p=img_p,nMisses=nMisses,status=status,_EXTRA=EXTRA_KEYWORDS
  status=-1
  if n_params() eq 3 then begin
    read_lst,p1,hdr_in,img_in,/struct
  ;img_in=readfits(p1,hdr_in)
  endif else begin
    img_in=p2
    hdr_in=p1
  endelse
  if not keyword_set(ni) then ni=1
  
  dsize=size(img_in)
  nx=dsize[1]
  ny=dsize[2]
  ;if (datatype(hdr_in) ne 'STR') then hdr_in=struct2fitshead(hdr_in)
  mb=mb_detect(img_in)
  
  if (mb[0] ne -1) then begin
    if n_elements(img_n) ne 0 && n_elements(img_p) ne 0 then begin
      if(datatype(img_n) eq 'STR') then begin
        read_lst,img_n,header,data_n
      ;data_n=readfits(img_n,header)
      endif else begin data_n=img_n
    endelse
    mbn=mb_detect(data_n)
    if(datatype(img_p) eq 'STR') then begin
      read_lst,img_p,header,data_p
    ;data_p=readfits(img_p,header)
    endif else begin data_p=img_p
  endelse
  mbp=mb_detect(data_p)
  mb1=setdifference(setdifference(mb,mbn),mbp)
  data[mb1]=(data_n[mb1]+data_p[mb1])/2
  mb=setdifference(mb,mb1)
endif

if (mb[0] ne -1) then begin
  img=dblarr(nx+4,ny+4)
  for j=1,ni do begin
    img[2:nx+1,2:ny+1]=img_in
    img[0,*]=img[4,*]
    img[1,*]=img[3,*]
    img[nx+2,*]=img[nx,*]
    img[nx+3,*]=img[nx-1,*]
    img[*,0]=img[*,4]
    img[*,1]=img[*,3]
    img[*,ny+2]=img[*,ny]
    img[*,ny+3]=img[*,ny-1]
    
    mb4=array_indices(img_in,mb)
    mb4=mb4+2
    mb3=mb4[1,*]*(nx+4)+mb4[0,*]
    
    
    for i=0,n_elements(mb3)-1 do begin
      re=[img[mb3[i]-2*nx-10],img[mb3[i]-2*nx-9],img[mb3[i]-2*nx-8],img[mb3[i]-2*nx-7],img[mb3[i]-2*nx-6],img[mb3[i]-nx-6],$
        img[mb3[i]-nx-2],img[mb3[i]-2],img[mb3[i]+2],img[mb3[i]+nx+2],img[mb3[i]+nx+6],img[mb3[i]+2*nx+6],img[mb3[i]+2*nx+7],$
        img[mb3[i]+2*nx+8],img[mb3[i]+2*nx+9],img[mb3[i]+2*nx+10]]
      img[mb3[i]]=median(re)
    endfor
    img_in=img[2:nx+1,2:ny+1]
  endfor
  
endif
mb=mb_detect(img_in)
if mb[0] ne -1 then begin nMisses=n_elements(mb)
endif else begin nMisses=0
endelse
endif else begin
  nMisses=0
endelse
img_out=img_in
;;;;alterd by Song Dechao on 2022.09.05
;hdr_in=fitshead2struct(hdr_in)
;;;;
;lst_gen_hdr_cal,hdr_in,img_out,outputHdr=outputHdr
totvals=hdr_in.TOTVALS;FXPAR(outputHdr,'TOTVALS')
hdr_in.MISSVALS=nMisses;fxaddpar,outputHdr,'MISSVALS', wcnt, 'Number of missing data values'
hdr_in.DATAVALS=totvals-nMisses;fxaddpar,outputHdr,'DATAVALS',totvals-wcnt,'Actual number of data values in image (pixels)'
hdr_in.PERCENTD=float(totvals-nMisses)/float(totvals)*100;fxaddpar,outputHdr,'PERCENTD',float(totvals-wcnt)/float(wcnt),'Percentage of actual/total image data number'

hdr_out=hdr_in
status=1
if n_params() eq 3 then begin
  p2=hdr_out
  p3=img_out
endif else begin
  p3=hdr_out
  p4=img_out
endelse
end
